--[[
	Armory Addon for World of Warcraft(tm).
	Revision: $Id: ArmorySpellBookFrame.lua,v 1.4, 2008-03-26 15:27:34Z, Maxim Baars$
    URL: http://www.wow-neighbours.com

	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
		You have an implicit licence to use this AddOn with these facilities
		since that is it's designated purpose as per:
		http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]] 

ARMORY_SPELLBOOK_PAGENUMBERS = {};

function ArmoryToggleSpellBook(bookType)
	-- If has no pet spells and is trying to open the corresponding book, then switch
	if ( not (Armory:HasPetSpells() and Armory:HasPetUI()) and bookType == BOOKTYPE_PET ) then
		bookType = BOOKTYPE_SPELL;
	end
	local isShown = ArmorySpellBookFrame:IsShown();
	HideUIPanel(ArmorySpellBookFrame);
	if ( (not isShown or (ArmorySpellBookFrame.bookType ~= bookType)) ) then
		ArmorySpellBookFrame.bookType = bookType;
        ArmoryCloseChildWindows();
        ShowUIPanel(ArmorySpellBookFrame);
    end
end

function ArmorySpellBookFrame_OnLoad()
    this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("SPELLS_CHANGED");
	this:RegisterEvent("PET_BAR_UPDATE");

	ArmorySpellBookFrame.bookType = BOOKTYPE_SPELL;
end

function ArmorySpellBookFrame_OnEvent()
    if ( not Armory:CanHandleEvents() ) then
        return;
    elseif ( event == "PLAYER_ENTERING_WORLD") then
        this:UnregisterEvent("PLAYER_ENTERING_WORLD");
    end
	Armory:Execute(ArmorySpellBookFrame_UpdateSpells);
end

function ArmorySpellBookFrame_UpdateSpells()
    Armory:SetSpells();
    ArmorySpellBookFrame_Update();
end

function ArmorySpellBookFrame_OnShow()
	-- Init page nums
	ARMORY_SPELLBOOK_PAGENUMBERS[1] = 1;
	ARMORY_SPELLBOOK_PAGENUMBERS[2] = 1;
	ARMORY_SPELLBOOK_PAGENUMBERS[3] = 1;
	ARMORY_SPELLBOOK_PAGENUMBERS[4] = 1;
	ARMORY_SPELLBOOK_PAGENUMBERS[5] = 1;
	ARMORY_SPELLBOOK_PAGENUMBERS[6] = 1;
	ARMORY_SPELLBOOK_PAGENUMBERS[7] = 1;
	ARMORY_SPELLBOOK_PAGENUMBERS[8] = 1;
	ARMORY_SPELLBOOK_PAGENUMBERS[BOOKTYPE_PET] = 1;
	
	-- Set to the first tab by default
	ArmorySpellBookSkillLineTab_OnClick(1);

	ArmorySpellBookFrame_Update(true);
end

function ArmorySpellBookFrame_Update(showing)
	local hasPetSpells, petToken = Armory:HasPetSpells();
	hasPetSpells = hasPetSpells and Armory:HasPetUI();
	if ( not hasPetSpells and ArmorySpellBookFrame.bookType == BOOKTYPE_PET ) then
        ArmorySpellBookFrame.bookType = BOOKTYPE_SPELL;
    end
    
	-- Hide all tabs
	ArmorySpellBookFrameTabButton1:Hide();
	ArmorySpellBookFrameTabButton2:Hide();
	ArmorySpellBookFrameTabButton3:Hide();
	
	-- Setup skillline tabs
	if ( showing ) then
		ArmorySpellBookSkillLineTab_OnClick(ArmorySpellBookFrame.selectedSkillLine);
	end

	local numSkillLineTabs = Armory:GetNumSpellTabs();
	local name, texture, offset, numSpells;
	local skillLineTab;
	for i = 1, MAX_SKILLLINE_TABS do
		skillLineTab = getglobal("ArmorySpellBookSkillLineTab"..i);
		if ( i <= numSkillLineTabs and ArmorySpellBookFrame.bookType == BOOKTYPE_SPELL ) then
			name, texture, offset, numSpells = Armory:GetSpellTabInfo(i);
			skillLineTab:SetNormalTexture(texture);
			skillLineTab.tooltip = name;
			skillLineTab:Show();

			-- Set the selected tab
			if ( ArmorySpellBookFrame.selectedSkillLine == i ) then
				skillLineTab:SetChecked(1);
			else
				skillLineTab:SetChecked(nil);
			end
		else
			skillLineTab:Hide();
		end
	end

	-- Setup tabs
	ArmorySpellBookFrame.petTitle = nil;
	if ( hasPetSpells ) then
		ArmorySpellBookFrame_SetTabType(ArmorySpellBookFrameTabButton1, BOOKTYPE_SPELL);
		ArmorySpellBookFrame_SetTabType(ArmorySpellBookFrameTabButton2, BOOKTYPE_PET, petToken);
	elseif ( ArmorySpellBookFrame.bookType == BOOKTYPE_PET ) then
		-- if has no pet spells but trying to show the pet spellbook close the window;
		HideUIPanel(ArmorySpellBookFrame);
		ArmorySpellBookFrame.bookType = BOOKTYPE_SPELL;
	end
	if ( ArmorySpellBookFrame.bookType == BOOKTYPE_SPELL ) then
		ArmorySpellBookTitleText:SetText(SPELLBOOK);
		if ( showing ) then
			PlaySound("igSpellBookOpen");
		end
	else
		ArmorySpellBookTitleText:SetText(ArmorySpellBookFrame.petTitle);
		-- Need to change to pet book open sound
		if ( showing ) then
			PlaySound("igAbilityOpen");
		end
	end
	ArmorySpellBookFrame_UpdatePages();
end

function ArmorySpellBookFrame_UpdatePages()
	local currentPage, maxPages = ArmorySpellBook_GetCurrentPage();
	if ( maxPages == 0 ) then
		ArmorySpellBookPrevPageButton:Disable();
		ArmorySpellBookNextPageButton:Disable();
        ArmorySpellBookPageText:SetText("");
		return;
	end
	if ( currentPage > maxPages ) then
		ARMORY_SPELLBOOK_PAGENUMBERS[ArmorySpellBookFrame.selectedSkillLine] = maxPages;
		currentPage = maxPages;
		ArmorySpellBook_UpdateSpells();
		if ( currentPage == 1 ) then
			ArmorySpellBookPrevPageButton:Disable();
		else
			ArmorySpellBookPrevPageButton:Enable();
		end
		if ( currentPage == maxPages ) then
			ArmorySpellBookNextPageButton:Disable();
		else
			ArmorySpellBookNextPageButton:Enable();
		end
	end
	if ( currentPage == 1 ) then
		ArmorySpellBookPrevPageButton:Disable();
	else
		ArmorySpellBookPrevPageButton:Enable();
	end
	if ( currentPage == maxPages ) then
		ArmorySpellBookNextPageButton:Disable();
	else
		ArmorySpellBookNextPageButton:Enable();
	end
	ArmorySpellBookPageText:SetFormattedText(PAGE_NUMBER, currentPage);
end

function ArmorySpellBookFrame_SetTabType(tabButton, bookType, token)
	if ( bookType == BOOKTYPE_SPELL ) then
		tabButton.bookType = BOOKTYPE_SPELL;
		tabButton:SetText(SPELLBOOK);
		tabButton.binding = "TOGGLESPELLBOOK";
	else
		tabButton.bookType = BOOKTYPE_PET;
		tabButton:SetText(getglobal("PET_TYPE_"..token));
		tabButton.binding = "TOGGLEPETBOOK";
		ArmorySpellBookFrame.petTitle = getglobal("PET_TYPE_"..token);
	end
	if ( ArmorySpellBookFrame.bookType == bookType ) then
		tabButton:Disable();
	else
		tabButton:Enable();
	end
	tabButton:Show();
end

function ArmorySpellBookFrame_OnHide()
	if ( this.bookType == BOOKTYPE_SPELL ) then
		PlaySound("igSpellBookClose");
	else
		-- Need to change to pet book close sound
		PlaySound("igAbilityClose");
	end
end
 
function ArmorySpellButton_OnEnter(self)
	local id = ArmorySpellBook_GetSpellID(self:GetID());
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	if ( id and Armory:SetSpell(id, ArmorySpellBookFrame.bookType) ) then
		self.UpdateTooltip = ArmorySpellButton_OnEnter;
	else
		self.UpdateTooltip = nil;
	end
end

function ArmorySpellButton_UpdateButton(button)
	if ( not ArmorySpellBookFrame.selectedSkillLine ) then
		ArmorySpellBookFrame.selectedSkillLine = 1;
	end
	local temp, texture, offset, numSpells = Armory:GetSpellTabInfo(ArmorySpellBookFrame.selectedSkillLine);
	ArmorySpellBookFrame.selectedSkillLineOffset = offset;
	local id = ArmorySpellBook_GetSpellID(button:GetID());
	local name = button:GetName();
	local iconTexture = getglobal(name.."IconTexture");
	local spellString = getglobal(name.."SpellName");
	local subSpellString = getglobal(name.."SubSpellName");
	local autoCastableTexture = getglobal(name.."AutoCastable");
	if ( id and (ArmorySpellBookFrame.bookType ~= BOOKTYPE_PET) and (id > (offset + numSpells)) ) then
		button:Disable();
		iconTexture:Hide();
		spellString:Hide();
		subSpellString:Hide();
		autoCastableTexture:Hide();
		getglobal(name.."NormalTexture"):SetVertexColor(1.0, 1.0, 1.0);
		return;
	else
		button:Enable();
	end
	local texture = Armory:GetSpellTexture(id, ArmorySpellBookFrame.bookType);
	local highlightTexture = getglobal(name.."Highlight");
	local normalTexture = getglobal(name.."NormalTexture");
	-- If no spell, hide everything and return
	if ( not texture or (strlen(texture) == 0) ) then
		iconTexture:Hide();
		spellString:Hide();
		subSpellString:Hide();
		autoCastableTexture:Hide();
		highlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square");
		normalTexture:SetVertexColor(1.0, 1.0, 1.0);
		return;
	end
    button.link = Armory:GetSpellLink(id, ArmorySpellBookFrame.bookType);

	local autoCastAllowed = Armory:GetSpellAutocast(id, ArmorySpellBookFrame.bookType);
	if ( autoCastAllowed ) then
		autoCastableTexture:Show();
	else
		autoCastableTexture:Hide();
	end

	local spellName, subSpellName = Armory:GetSpellName(id, ArmorySpellBookFrame.bookType);
	local isPassive = IsPassiveSpell(id, ArmorySpellBookFrame.bookType);
	if ( isPassive ) then
		normalTexture:SetVertexColor(0, 0, 0);
		highlightTexture:SetTexture("Interface\\Buttons\\UI-PassiveHighlight");
		--subSpellName = PASSIVE_PARENS;
		spellString:SetTextColor(PASSIVE_SPELL_FONT_COLOR.r, PASSIVE_SPELL_FONT_COLOR.g, PASSIVE_SPELL_FONT_COLOR.b);
	else
		normalTexture:SetVertexColor(1.0, 1.0, 1.0);
		highlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square");
		spellString:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
	end
	iconTexture:SetTexture(texture);
	spellString:SetText(spellName);
	subSpellString:SetText(subSpellName);
	if ( subSpellName ~= "" ) then
		spellString:SetPoint("LEFT", button, "RIGHT", 4, 4);
	else
		spellString:SetPoint("LEFT", button, "RIGHT", 4, 2);
	end

	iconTexture:Show();
	spellString:Show();
	subSpellString:Show();
end

function ArmoryPrevPageButton_OnClick()
	local pageNum = ArmorySpellBook_GetCurrentPage() - 1;
	if ( ArmorySpellBookFrame.bookType == BOOKTYPE_SPELL ) then
		PlaySound("igAbiliityPageTurn");
		ARMORY_SPELLBOOK_PAGENUMBERS[ArmorySpellBookFrame.selectedSkillLine] = pageNum;
	else
		ArmorySpellBookTitleText:SetText(ArmorySpellBookFrame.petTitle);
		-- Need to change to pet book pageturn sound
		PlaySound("igAbiliityPageTurn");
		ARMORY_SPELLBOOK_PAGENUMBERS[BOOKTYPE_PET] = pageNum;
	end
	ArmorySpellBook_UpdatePageArrows();
	ArmorySpellBookPageText:SetFormattedText(PAGE_NUMBER, pageNum);
	ArmorySpellBook_UpdateSpells();
end

function ArmoryNextPageButton_OnClick()
	local pageNum = ArmorySpellBook_GetCurrentPage() + 1;
	if ( ArmorySpellBookFrame.bookType == BOOKTYPE_SPELL ) then
		PlaySound("igAbiliityPageTurn");
		ARMORY_SPELLBOOK_PAGENUMBERS[ArmorySpellBookFrame.selectedSkillLine] = pageNum;
	else
		ArmorySpellBookTitleText:SetText(ArmorySpellBookFrame.petTitle);
		-- Need to change to pet book pageturn sound
		PlaySound("igAbiliityPageTurn");
		ARMORY_SPELLBOOK_PAGENUMBERS[BOOKTYPE_PET] = pageNum;
	end
	ArmorySpellBook_UpdatePageArrows();
	ArmorySpellBookPageText:SetFormattedText(PAGE_NUMBER, pageNum);
	ArmorySpellBook_UpdateSpells();
end

function ArmorySpellBookSkillLineTab_OnClick(id)
	ArmorySpellBookFrame.selectedSkillLine = id or this:GetID();
	local name, texture, offset, numSpells = Armory:GetSpellTabInfo(ArmorySpellBookFrame.selectedSkillLine);
	ArmorySpellBookFrame.selectedSkillLineOffset = offset;
	ArmorySpellBookFrame.selectedSkillLineNumSpells = numSpells;
	ArmorySpellBook_UpdatePageArrows();
	ArmorySpellBookFrame_Update();
	ArmorySpellBookPageText:SetFormattedText(PAGE_NUMBER, ArmorySpellBook_GetCurrentPage());
	ArmorySpellBook_UpdateSpells();
end

function ArmorySpellBook_GetSpellID(id)
	if ( ArmorySpellBookFrame.bookType == BOOKTYPE_PET ) then
		return id + (SPELLS_PER_PAGE * (ARMORY_SPELLBOOK_PAGENUMBERS[BOOKTYPE_PET] - 1));
	elseif ( ArmorySpellBookFrame.selectedSkillLineOffset ) then
		return id + ArmorySpellBookFrame.selectedSkillLineOffset + ( SPELLS_PER_PAGE * (ARMORY_SPELLBOOK_PAGENUMBERS[ArmorySpellBookFrame.selectedSkillLine] - 1));
	end
end

function ArmorySpellBook_UpdatePageArrows()
	local currentPage, maxPages = ArmorySpellBook_GetCurrentPage();
	if ( currentPage == 1 ) then
		ArmorySpellBookPrevPageButton:Disable();
	else
		ArmorySpellBookPrevPageButton:Enable();
	end
	if ( maxPages == 0 or currentPage == maxPages ) then
		ArmorySpellBookNextPageButton:Disable();
	else
		ArmorySpellBookNextPageButton:Enable();
	end
end

function ArmorySpellBook_GetCurrentPage()
	local currentPage, maxPages;
	local numPetSpells = Armory:HasPetSpells();
	if ( numPetSpells and Armory:HasPetUI() and ArmorySpellBookFrame.bookType == BOOKTYPE_PET ) then
		currentPage = ARMORY_SPELLBOOK_PAGENUMBERS[BOOKTYPE_PET];
		maxPages = ceil(numPetSpells / SPELLS_PER_PAGE);
	else
		currentPage = ARMORY_SPELLBOOK_PAGENUMBERS[ArmorySpellBookFrame.selectedSkillLine];
		local name, texture, offset, numSpells = Armory:GetSpellTabInfo(ArmorySpellBookFrame.selectedSkillLine);
		if ( numSpells ) then
		    maxPages = ceil(numSpells / SPELLS_PER_PAGE);
		else
		    maxPages = 0;
		end
	end
	return currentPage, maxPages;
end

function ArmorySpellBook_UpdateSpells()
    for i = 1, SPELLS_PER_PAGE do
       ArmorySpellButton_UpdateButton(getglobal("ArmorySpellButton"..i));
    end
end