--[[
	Armory Addon for World of Warcraft(tm).
	Revision: $Id: ArmoryTitan.lua,v 1.6, 2008-04-08 17:08:55Z, Maxim Baars$
    URL: http://www.wow-neighbours.com

	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
		You have an implicit licence to use this AddOn with these facilities
		since that is it's designated purpose as per:
		http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]] 

local TitanPanelButton_SetButtonIcon_Hook = TitanPanelButton_SetButtonIcon;

function TitanPanelButton_SetButtonIcon(id, ...)
    if ( Armory.dbLoaded ) then
        Armory:SetPortraitTexture(TitanPanelArmoryButtonIcon, "player");
    end
    return TitanPanelButton_SetButtonIcon_Hook(id, ...);
end


function TitanPanelArmoryButton_OnLoad()
	this.registry = { 
		id = ARMORY_ID,
		version = Armory.version,
		menuText = ARMORY_TITLE,
		category = "Information",
		tooltipTitle = ARMORY_TITLE,
		tooltipTextFunction = "TitanPanelArmoryButton_GetTooltipText",
		iconWidth = 16,
		savedVariables = { ShowIcon = true }
	};
	
	table.insert(TITAN_PANEL_NONMOVABLE_PLUGINS, ARMORY_ID);
end

function TitanPanelArmoryButton_OnClick(button)
	if ( button == "LeftButton" ) then
		Armory:Toggle();
	end
end
 
function TitanPanelArmoryButton_GetTooltipText()
    local realm, character = Armory:GetPaperDollLastViewed();

    return ARMORY_TOOLTIP1.."\t"..HIGHLIGHT_FONT_COLOR_CODE..character.."\n"..
           ARMORY_TOOLTIP2.."\t"..HIGHLIGHT_FONT_COLOR_CODE..realm.."\n"..
           GREEN_FONT_COLOR_CODE..ARMORY_TOOLTIP_HINT1.."\n"..
           GREEN_FONT_COLOR_CODE..ARMORY_TOOLTIP_HINT2;
end

function TitanPanelRightClickMenu_PrepareArmoryMenu()
    local info;
    
    TitanPanelRightClickMenu_AddTitle(TitanPlugins[ARMORY_ID].menuText);

    ArmoryTitan_AddButton("ARMORY_CMD_SET_SEARCHALL");
    ArmoryTitan_AddButton("ARMORY_CMD_SET_LASTVIEWED");
    ArmoryTitan_AddButton("ARMORY_CMD_SET_PERCHARACTER");
    ArmoryTitan_AddButton("ARMORY_CMD_SET_SHOWALTEQUIP", true);
    ArmoryTitan_AddButton("ARMORY_CMD_SET_SHOWUNEQUIP");
    ArmoryTitan_AddButton("ARMORY_CMD_SET_SHOWEQCTOOLTIPS");
    ArmoryTitan_AddButton("ARMORY_CMD_SET_SHOWITEMCOUNT");
    ArmoryTitan_AddButton("ARMORY_CMD_CHECK");
    ArmoryTitan_AddButton("ARMORY_CMD_RESET_FRAME");

    TitanPanelRightClickMenu_AddSpacer();
    TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, ARMORY_ID, TITAN_PANEL_MENU_FUNC_HIDE);
end

function ArmoryTitan_AddButton(label, closeMenu)
    local entry = Armory.options[label];
    local info = {};
    
    info.text = getglobal(label.."_MENUTEXT");
    info.tooltipTitle = info.text;
    info.tooltipText = getglobal(label.."_TOOLTIP") or Armory:Proper(getglobal(label.."_TEXT"));
    if ( entry.type == "toggle" ) then
        info.getFunc = entry.get;
        info.setFunc = entry.set;
        info.func = function() info.setFunc(not info.getFunc()) end;
        info.checked = entry.get();
        info.keepShownOnClick = not closeMenu;
    else
        info.runFunc = entry.run;
        info.func = function() info.runFunc() end;
    end
    if ( entry.disabled ) then
        info.disabled = entry.disabled();
    end
    
    UIDropDownMenu_AddButton(info);
end
